/* NSString_Lines.m created by eb on Mon 19-Apr-1999 */

#import "NSString_Lines.h"
#import "NSValue_Ranges.h"

#define INRANGE(a,b) (a.location + a.length <= b)
#define EQUALRANGE(a,b) ((a.location == b.location) && (a.length == b.length))

@implementation NSString(Lines)

- (NSRange) rangeOfLine: (unsigned int) lineNumber
{
  NSRange lineRange,lastRange;
  int lineCount, myLength;

  myLength = [self length];
  lineCount = 0;

  lastRange.location = 0;
  lastRange.length = 0;

  // find line
  lineRange = [self lineRangeForRange: lastRange];
  while((lineCount < lineNumber) && INRANGE(lineRange,myLength)
        && (!EQUALRANGE(lastRange,lineRange))){
    lastRange = lineRange;
    lineRange.location += lineRange.length;
    lineRange.length = 0;
    lineRange = [self lineRangeForRange: lineRange];
    lineCount++;
  }

  // if still in range
  if(INRANGE(lineRange,myLength && (!EQUALRANGE(lastRange,lineRange))) ){
    return lineRange;
  }

  lineRange.location = NSNotFound;
  lineRange.length = 0;
  return lineRange;
}

- (NSArray *) arrayOfLineRangesInRange: (NSRange) fullRange
{
  NSRange lineRange,lastRange;
  NSMutableArray *array = [NSMutableArray array];
  BOOL lastRangeEqualsLineRange = NO;

  if(!INRANGE(fullRange,[self length])){
    [NSException raise: NSRangeException
                format: @"%@: range outside of text",
      NSStringFromSelector(_cmd)];
  }

  lastRange.location = fullRange.location;
  lastRange.length = 0;

  // find line
  lineRange = [self lineRangeForRange: lastRange];
  while(lineRange.location <= (fullRange.location + fullRange.length)
        && !lastRangeEqualsLineRange){
    [array addObject: [NSValue valueWithRange: lineRange]];
    lastRange = lineRange;
    lineRange.location += lineRange.length;
    lineRange.length = 0;
    lineRange = [self lineRangeForRange: lineRange];
    lastRangeEqualsLineRange = EQUALRANGE(lastRange,lineRange);
  }

  return array;
}

@end
